#ifndef _INA219
#define _INA219

#include "ForteSensor.hpp"
#include "Message.hpp"
#include "Pinout.hpp"
#include "Wire.h"
#include "esp_log.h"
#include <INA219_WE.h>
#include <MeasurementTypes.h>


struct out_data_ina219 {
  Measurement shuntVoltage_mV;
  Measurement loadVoltage_V;
  Measurement busVoltage_V;
  Measurement current_mA;
  Measurement power_mW;
  bool ina219_overflow = false;
};

struct pin_setup_ina219 {
  uint8_t sda;
  uint8_t scl;
};

class ForteINA219: public ForteSensor<out_data_ina219> {
 public:
  void setup() override;
  out_data_ina219 readData() override;
  std::list<Message> buildMessages() override;
  [[nodiscard]] SensorInformation getSensorInformation() const override;
  void setPinSetup(pin_setup_ina219 pinSetup);

 private:
  INA219_WE ina219;
  out_data_ina219 data;
  pin_setup_ina219 pinSetup{21, 22};
  const SensorInformation
      sensorInformation{HardwareName::INA219, SensorProtocol::I2C};
  const uint8_t INA219_BASE_ADDRESS = 0x40;

};

#endif