#include "rs485.hpp"
#include "SentecRainGaugeSensor.h"
#include "SentecSolarRadiationSensor.h"
// RS485 control
//#define RS485Serial Serial2

#define RX_PIN 14 // Serial Receive pin
#define TX_PIN 15 // Serial Transmit pin

#define RE_DE_PIN 19 // Line to pull high or low to receive or send data from RS485

#define POWER_SWITCH_PIN_12V 12
#define POWER_SWITCH_PIN_5V 13

static const char *TAG = "RS485";

// Configure sensors
SEM228A solarSensor(1, RE_DE_PIN);
SEM404 rainGauge = SEM404(2, RE_DE_PIN);

void ForteRS485::setup() {
    RS485Serial = RS485HardwareSerial::getInstance().getRS485Serial();
    // configure the pins to be output only
    pinMode(RE_DE_PIN, OUTPUT);
    pinMode(POWER_SWITCH_PIN_12V, OUTPUT);
    pinMode(POWER_SWITCH_PIN_5V, OUTPUT);
    RS485Serial->begin(4800, SERIAL_8N1, TX_PIN, RX_PIN);
}

void ForteRS485::teardown() {
    RS485Serial->end();
}

out_data_rs485 ForteRS485::readData() {
    powerOnRS485Sensors();

    out_data_rs485 output{};
    output.solar = solarSensor.readData();
    output.precipitation = rainGauge.readData();

    powerOffRS485Sensors();
    return output;
}
void ForteRS485::powerOffRS485Sensors() {
    digitalWrite(POWER_SWITCH_PIN_12V, LOW);
    digitalWrite(POWER_SWITCH_PIN_5V, LOW);

    gpio_hold_en((gpio_num_t)POWER_SWITCH_PIN_12V);
    gpio_hold_en((gpio_num_t)POWER_SWITCH_PIN_5V);
}
void ForteRS485::powerOnRS485Sensors() { // Power on sensor
    digitalWrite(POWER_SWITCH_PIN_12V, HIGH);
    digitalWrite(POWER_SWITCH_PIN_5V, HIGH); // Wait for sensors to power up
    // TODO minimize delay
    delay(100);
}

std::list<Message> ForteRS485::buildMessages() {
    std::list<Message> messages;
    out_data_rs485 output = readData();

    Measurement solarRadiation{output.solar.solarRadiation, 1, NO_I2C_ADDRESS, MeasurementType::SOLAR_RADIATION,
                               output.solar.solarError};
    Measurement precipitation{output.precipitation.precipitation, 2, NO_I2C_ADDRESS, MeasurementType::PRECIPITATION,
                              output.precipitation.precipitationError};

    messages.emplace_back(solarRadiation, sensorInformation, Time::getInstance().getEpochSeconds());
    messages.emplace_back(precipitation, sensorInformation, Time::getInstance().getEpochSeconds());

    return messages;
}

SensorInformation ForteRS485::getSensorInformation() const {
    return sensorInformation;
}