#ifndef _RS485
#define _RS485

#include "ForteSensor.hpp"
#include "Message.hpp"
#include "SentecSensorsRS485.h"
#include <MeasurementTypes.h>
#include <RS485HardwareSerial.h>
#include <SentecRainGaugeSensor.h>
#include <SentecSolarRadiationSensor.h>

/**
 * @brief Struct to hold all the data from the RS485 sensors
 */

struct out_data_rs485 {
    out_data_solar_radiation solar;
    out_data_rain_gauge precipitation;
};

/**
 * @brief Class to read data from both the Sentec solar radiation sensor and the Sentec rain gauge sensor at once
 * This class is required to be instantiated and setup in order to use RS485 communication.
 */
class ForteRS485 : public ForteSensor<out_data_rs485> {
  public:
    void setup() override;
    out_data_rs485 readData() override;
    std::list<Message> buildMessages() override;
    [[nodiscard]] SensorInformation getSensorInformation() const override;
    void teardown();

    /**
     * @brief Power on the RS485 sensors
     */
    static void powerOnRS485Sensors();

    /**
     * @brief Power off the RS485 sensors
     */
    static void powerOffRS485Sensors();

  private:
    std::shared_ptr<HardwareSerial> RS485Serial;

    const SensorInformation sensorInformation{HardwareName::RS485, SensorProtocol::RS485};
};
#endif