//
// Created by zoe on 12/20/22.
//

#ifndef HOST_CENTRAL_MAST_CONNECTIONMANAGER_H
#define HOST_CENTRAL_MAST_CONNECTIONMANAGER_H

#define TINY_GSM_MODEM_SIM7000

#include "Definitions.h"
#include "GPRSCredentials.h"
#include "LTEConnectionException.hpp"
#include "Mode.h"
#include "ModemFunctionalityLevel.h"
#include "NetworkMode.h"
#include "RequestInformation.h"
#include "StreamDebugger.h"
#include <TinyGsmClient.h>
#include <utility>

class ConnectionManager {

  public:
    struct ModemPins {
        int dtr;
        int tx;
        int rx;
        int pwr;
    };

  private:
    TinyGsm &modem;
    const GPRSCredentials credentials;
    ModemPins modemPins;

  public:
    ConnectionManager(TinyGsm &modem, GPRSCredentials credentials, ModemPins modemPins);

    std::string connect(RequestInformation requestInformation);

    void enableGPS();

    void disableGPS();

    void initModem();

    void restartModem();

    void setNetworkMode(NetworkMode networkMode = NetworkMode::LTE_ONLY);

    void setPreferredMode(Mode mode = Mode::CAT_M);

    void setModemFunctionalityLevel(ModemFunctionalityLevel functionalityLevel = MINIMAL);

    enum BandMode { BAND_CAT_M, BAND_NB_IoT };

    void setBand(BandMode bandMode, int band);

    bool gprsConnect();
    bool gprsDisconnect();

    bool isNetworkConnected();

    void modemPowerOn();

    void modemPowerOff();

    void unlockSimCard();
    bool waitForNetwork();

    void logModemInformation();
    bool isGprsConnected();
};

#endif // HOST_CENTRAL_MAST_CONNECTIONMANAGER_H
