//
// Created by zoe on 1/24/23.
//

#include "SDCardLogger.h"

namespace SDCardLogger {

    char log_print_buffer[512];
    bool printToSerial = false;

    void printDebugToSerial(bool printToSerialAsWell) {
        printToSerial = printToSerialAsWell;
    }
    int vprintf_into_sd(const char *szFormat, va_list args) {
        auto timeSinceBootMs = esp_timer_get_time() / 1000;
        String logstring = String(timeSinceBootMs) + ", ";
        // print current core
        logstring += "Core: ";
        logstring += xPortGetCoreID();
        logstring += ", ";
        logstring += "[" + rtc.getDateTime() + "] ";
        logstring += szFormat;
        if (!SDUtilities::isSDAvailable()) {
            if (printToSerial) {
                logstring += " (SD card not available)\n";

                vprintf(logstring.c_str(), args);
            }
            return 1;
        }

        // write evaluated format string into buffer
        int ret = vsnprintf(log_print_buffer, sizeof(log_print_buffer), logstring.c_str(), args);

        String date = rtc.getDate();
        String filename = "/log_" + date + ".txt";

        // output is now in buffer. write to file.
        if (ret >= 0) {
            if (!SD.exists(filename)) {
                File writeLog = SD.open(filename, FILE_WRITE);
                if (!writeLog) {
                    Serial.println("Couldn't open " + filename + " for writing");
                    SDUtilities::setSDAvailable(false);
                    return 1;
                }
                delay(50);
                writeLog.close();
            }

            File logFile = SD.open(filename, FILE_APPEND);

            // debug output
            if (printToSerial) {
                vprintf(logstring.c_str(), args);
            }
            logFile.write((uint8_t *)log_print_buffer, (size_t)ret);
            // to be safe in case of crashes: flush the output
            logFile.flush();
            logFile.close();
        }
        return ret;
    }

} // namespace SDCardLogger
