//
// Created by zoe on 1/9/23.
//

#ifndef HOST_CENTRAL_MAST_UTILITIES_H
#define HOST_CENTRAL_MAST_UTILITIES_H

#include "ArduinoJson.h"
#include "MessageType.h"
#include "SD.h"
#include "SDCardException.h"
#include "SDSetupException.h"
#include "WiFi.h"
#include <Arduino.h>
#include <ClientDataPackage.hpp>
#include <Definitions.h>
#include <WString.h>
#include <esp_now.h>
#include <list>

namespace SDUtilities {
    File openDirectory(const String &dirname);
    std::list<String> getFilesInDirectory(const String &dirname);
    std::optional<String> getFirstFileNameInDirectory(const String &dirname);
    std::optional<String> getLastFileInDirectory(const String &dirname);
    void writeFile(const String &messageToBeSend, const String &filePath);
    String readFile(const String &filePath);
    void createDirectory(const String &dirname);
    void setupSDCard(int MISO, int MOSI, int SCLK, int CS);
    void saveStringToSDCard(const std::string &dataString);
    void setSDAvailable(bool sdAvailable);
    bool isSDAvailable();
} // namespace SDUtilities

namespace ESPUtilities {

    String getMacAddressAsString(const uint8_t *mac);
    void checkPeerExistence(const uint8_t *mac);
    void sendResponse(const uint8_t *mac);
} // namespace ESPUtilities

void turnOffLEDs();
String getMacAddressAsString(const uint8_t *mac);
String documentToLineProtocolString(const DynamicJsonDocument &doc);
String dataPackageToLineProtocolString(const ClientDataPackage &dataPackage, const String &clientMacAddress);
DynamicJsonDocument parseReceivedJsonData(char *data);
String documentToServerReadableString(const DynamicJsonDocument &doc);
String dataPackageToServerReadableString(ClientDataPackage clientDataPackage, const String &clientMacAddress);

#endif // HOST_CENTRAL_MAST_UTILITIES_H
