#include "ESP32Time.h"
#include <Arduino.h>
#include <ArduinoJson.h>
#include <WiFi.h>
#include <esp_log.h>
#include <esp_now.h>

static const std::string TAG = "MAIN";
static const std::string TAG_ESPNOW = "ESPNOW";
static const std::string TAG_GSM = "GSM";

ESP32Time rtc;
TaskHandle_t ESPNOWTask;

void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
	// go to sleep
}

String getMacAddressAsString(const uint8_t *mac);
DynamicJsonDocument parseReceivedJsonData(char *data);

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
	esp_log_write(ESP_LOG_INFO, TAG_ESPNOW.c_str(), "Message recieved\n");
	// copy received data to a char array
	char data[len];
	memcpy(data, incomingData, len);
	esp_log_write(ESP_LOG_DEBUG, TAG_ESPNOW.c_str(), "Raw received Data: %s\n", data);

	DynamicJsonDocument doc = parseReceivedJsonData(data);

	String macAddress = getMacAddressAsString(mac);

	// add timestamp and mac address
	doc["timestamp"] = rtc.getEpoch();
	doc["clientMac"] = macAddress;

	// serialize json document again
	std::string dataString{};
	serializeJson(doc, dataString);
}

DynamicJsonDocument parseReceivedJsonData(char *data)
{
	DynamicJsonDocument doc(250);
	auto error = deserializeJson(doc, data);
	if (error) {
		esp_log_write(ESP_LOG_ERROR, TAG_ESPNOW.c_str(), "Error while parsing json: %s\n", error.f_str());
		// TODO error handling
	}
	return doc;
}
String getMacAddressAsString(const uint8_t *mac)
{
	String macAddress;
	for (int i = 0; i < 6; i++) {
		macAddress += String(mac[i], HEX);
	}
	esp_log_write(ESP_LOG_DEBUG, TAG_ESPNOW.c_str(), "MAC: %s\n", macAddress.c_str());
	return macAddress;
}

[[noreturn]] void ESPNOWReceiveTask(void *parameter)
{
	esp_log_write(ESP_LOG_DEBUG, TAG_ESPNOW.c_str(), "ESPNOWReceiveTask started on core %d\n", xPortGetCoreID());

	WiFi.mode(WIFI_STA);

	esp_log_write(ESP_LOG_DEBUG, TAG_ESPNOW.c_str(), "Initialising ESPNow...\n");

	if (esp_now_init() != ESP_OK) {
		// initialization failed
		esp_log_write(ESP_LOG_ERROR, TAG_ESPNOW.c_str(), "Initialising ESPNow FAILED\n");
		exit(ESP_FAIL);
	}

	esp_log_write(ESP_LOG_DEBUG, TAG_ESPNOW.c_str(), "Initialising ESPNow SUCCESS\n");

	esp_now_register_recv_cb(on_data_recv);

	while (true) {
	}
}

void setup()
{
	// Set console baud rate
	Serial.begin(115200);
	delay(10);

	//	https://stackoverflow.com/questions/60442350/arduinos-esp-log-set-vprintf-does-not-work-on-esp32
	esp_log_level_set("*", ESP_LOG_VERBOSE);
	esp_log_write(ESP_LOG_DEBUG, TAG.c_str(), "%s", WiFi.macAddress().c_str());

	delay(1000);

	// create ESPNOWReceiveTask. TODO: Until the UTC time is not synced, this will not add the correct time. If we
	// TODO: create the task after the time is synced, no messages will be received until synchronization is done
	xTaskCreatePinnedToCore(ESPNOWReceiveTask,   /* Function to implement the task */
	                        "ESPNOWReceiveTask", /* Name of the task */
	                        10000,               /* Stack size in words */
	                        nullptr,             /* Task input parameter */
	                        0,                   /* Priority of the task */
	                        &ESPNOWTask,         /* Task handle. */
	                        0);                  /* Core where the task should run */

	// Restart takes quite some time
	// To skip it, call init() instead of restart()
	esp_log_write(ESP_LOG_DEBUG, TAG_GSM.c_str(), "Initializing modem...\n");
}

void loop() {}
